package com.jnssd.config;

import io.swagger.v3.oas.models.Components;
import io.swagger.v3.oas.models.ExternalDocumentation;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.oas.models.info.Contact;
import io.swagger.v3.oas.models.info.Info;
import io.swagger.v3.oas.models.info.License;
import io.swagger.v3.oas.models.security.*;
import org.springdoc.core.GroupedOpenApi;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * <h3>spring-boot-openapi</h3>
 * <p></p>
 *
 * @author zxj
 * @since 2023-10-16 17:41:17
 */
@Configuration
@EnableConfigurationProperties(SwaggerProperties.class)
public class SpringDocConfig {
    @Bean
    public OpenAPI openAPI(SwaggerProperties properties) {
        return new OpenAPI()
                .info(createInfo(properties))
                // .externalDocs(new ExternalDocumentation()
                //         .description("SpringDoc API功能 演示")
                //         .url("http://127.0.0.1:" + port + "/"))
                .components(new Components()
                        .addSecuritySchemes("basicScheme", new SecurityScheme().type(SecurityScheme.Type.HTTP).scheme("basic").bearerFormat("JWT"))
                        .addSecuritySchemes("oauth2", securitySchemeOauth2Password()))
                .addSecurityItem(new SecurityRequirement().addList("basicScheme")
                        .addList("oauth2"));
    }

    /**
     * 文档信息
     * @param properties 配置信息
     */
    Info createInfo(SwaggerProperties properties){
        return  new Info()
                .title(properties.getTitle())
                .description(properties.getDescription())
                .termsOfService(properties.getTermsOfServiceUrl())
                .contact(new Contact().name(properties.getContact().getName()).url(properties.getContact().getUrl()).email(properties.getContact().getEmail()))
                .license(new License().name("Apache 2.0").url(properties.getLicense()))
                .version(properties.getVersion());
    }

    @Bean
    public GroupedOpenApi menuApi() {
        return GroupedOpenApi.builder()
                .addOpenApiCustomiser(openApi -> openApi.info(new Info().title("菜单 API")
                        .description("菜单 API 演示")
                        .version("v1.0.0")
                        .license(new License().name("Apache 2.0").url("jnssd.com"))
                        .contact(new Contact().name("zxj").email("zxj@163.com").url("jnssd.com"))
                        .termsOfService("jnssd")
                ).externalDocs(new ExternalDocumentation()
                        .description("菜单 API功能 演示").url("jnssd.com")))
                .group("menu")
                .pathsToMatch("/menu/**")
                .addOperationCustomizer((operation, handlerMethod) -> {
                    operation.addSecurityItem(new SecurityRequirement().addList("basicScheme").addList("oauth2"));
                    return operation;
                })
                .build();
    }
    //
    // @Bean
    // public GroupedOpenApi userApi() {
    //     return GroupedOpenApi.builder()
    //             .group("user")
    //             .pathsToMatch("/user/**")
    //             .build();
    // }

    // @Bean
    // public OpenAPI customOpenAPI() {
    //     SecurityScheme oauth2Scheme = new SecurityScheme()
    //             .type(SecurityScheme.Type.OAUTH2)
    //             .flows(new OAuthFlows()
    //                     .password(new OAuthFlow()
    //                             .tokenUrl("/oauth/token")
    //                             .scopes(new Scopes()
    //                                     .addString("read_scope", "Grants read access")
    //                                     .addString("write_scope", "Grants write access")
    //                                     .addString("admin_scope", "Grants read write and delete access")
    //                             )
    //                     )
    //             );
    //
    //     Components components = new Components().addSecuritySchemes("OAuth2", oauth2Scheme);
    //     SecurityRequirement securityRequirement = new SecurityRequirement().addList("OAuth2");
    //
    //     return new OpenAPI()
    //             .components(components)
    //             .addSecurityItem(securityRequirement);
    // }

    /**
     * 配置oauth2密码模式
     *
     * @return
     */
    SecurityScheme securitySchemeOauth2Password() {
        return new SecurityScheme()
                .type(SecurityScheme.Type.OAUTH2)
                .flows(new OAuthFlows()
                        .password(new OAuthFlow()
                                .tokenUrl("/oauth/token")
                                .scopes(new Scopes()
                                        .addString("read_scope", "Grants read access")
                                        .addString("write_scope", "Grants write access")
                                        .addString("admin_scope", "Grants read write and delete access")
                                )
                        )
                );
    }
}
